/*

Politecnico di Torino - Fondamenti di Informatica (06AZN)

una possibile soluzione del tema d'esame del 28/2/2012

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* massimo numero di caratteri di una data gg/mm/aaaa */
#define MAXDATA 10

/* massimo numero di caratteri di un orario hh:mm */
#define MAXORA 5

/* ogni numero telefonico ha al massimo 12 cifre */
#define MAXNUMERO 12

/* dimensione massima di una riga di input */
#define MAXBC 127

/* al massimo devo considerare 100 numeri telefonici diversi */
#define MAXNUMTELEF 100

  
/* struttura dati per memorizzare il numero di telefonate fatte ad uno specifico destinatario */

struct _telefonate {
  char numero[MAXNUMERO+1];
  int num_chiamate;
};

struct _telefonate telefonate[MAXNUMTELEF]; 


/* numero di elementi presente in telefonate[] */

unsigned n = 0;


/*
funzione per aggiungere un numero all'elenco (se non ancora presente)
o per aggiungere una telefonata (se numero gia' presente)
*/

void aggiungi_telefonata (char numero[])
{
  unsigned i;

  /* cerco il numero: se e' gia' presente incremento il numero di chiamate fattegli */

  for (i=0; i<n; i++)
  {
    if (strcmp(telefonate[i].numero,numero) == 0)
    {
      telefonate[i].num_chiamate++; 
      return;
    }
  }

  /* numero non presente, lo aggiungo in coda all'elenco */
  /* NOTA: si potrebbe controllare se (n < MAXNUMTELEF) */
  /* ma il testo garantiva che non si avranno mai piu' di 100 numeri diversi */

  strcpy (telefonate[n].numero, numero);
  telefonate[n].num_chiamate = 1;
  n++; 

}     


/*
funzione per cercare il numero telefonico col massimo numero di chiamate
*/

unsigned cerca_max_num_chiamate ()
{
  unsigned i;
  unsigned max_num_chiamate = 0;
  unsigned indice_max_num_chiamate = 0;

  /* cerco il numero telefonico col massimo numero di chiamate */

  for (i=0; i<n; i++)
  {
    if (telefonate[i].num_chiamate > max_num_chiamate)
    {
      max_num_chiamate = telefonate[i].num_chiamate;
      indice_max_num_chiamate = i;
    }
  }

  /* restituisco l'indice del numero telefonico col massimo numero di chiamate */

  return indice_max_num_chiamate;
}     



int main (int argc, char *argv[])
{

  /* lettura da file */
  FILE *afh;
  char buf[MAXBC+1];
  unsigned n_telefonate = 0;
  
  /* dati di una telefonata */
  char t_data[MAXDATA+1];
  char t_ora[MAXORA+1];
  char t_numero[MAXNUMERO+1];
  int t_durata;
  float t_costo;

  /* durata totale di tutte le telefonate */
  int durata_totale= 0;

  /* controllo gli argomenti sulla linea di comando */

  if (argc != 2) {
    fprintf (stderr, "errore - inserire il nome del file delle telefonate\n");
    exit (1);
  }

  /* apro il file delle telefonate */

  if ((afh=fopen(argv[1],"r")) == NULL) {
    fprintf (stderr, "errore - impossibile aprire il file %s in lettura\n", argv[1]);
    exit (1);
  }

  /* lettura delle telefonate (una per riga) dal file */

  while (fgets(buf,MAXBC,afh) != NULL)
  {
    if ((sscanf (buf, "%s %s %s %d %f\n", t_data, t_ora, t_numero, &t_durata, &t_costo) != 5))
    {
      /* questa riga contiene dati non corretti; viene, pertanto, ignorata */
      fprintf (stderr, "errore - riga sbagliata - ignorata:\n%s\n", buf);
      continue;
    }

    /* se chiamata valida, incremento il totale e la aggiungo in telefonate[] */

    n_telefonate++;

    aggiungi_telefonata (t_numero);

    /* se chiamata non gratuita, aggiungo la sua durata al totale */

    if (t_costo > 0)
      durata_totale += t_durata;
  }

  if (n_telefonate < 1)
    printf ("avviso - archivio %s vuoto - nessuna telefonata valida\n", argv[1]);
  else
  {
    /* esprimo la durata totale in ore, minuti e secondi */

    int ore = durata_totale / 3600; 
    int minuti = (durata_totale % 3600) / 60;
    int secondi = (durata_totale % 3600) % 60;
    /* oppure
    int minuti = (durata_totale - ore * 3600) / 60;
    int secondi = durata_totale - ore * 3600 - minuti * 60;
    */
    printf ("durata totale chiamate non gratuite: %d ore, %d minuti, %d secondi\n", ore, minuti, secondi);

    /* cerco e presento il numero piu' usato */

    unsigned imax = cerca_max_num_chiamate();
    printf ("Numero maggiormente chiamato: %s (%d volte)\n",
      telefonate[imax].numero, telefonate[imax].num_chiamate);
  }

  /* chiusura del file e termine del programma */

  if (fclose(afh))
    fprintf (stderr, "avviso - fallita chiusura del file %s\n", argv[1]);

  return 0;
}
